import { Component, Vue, Watch } from 'vue-property-decorator';
import NavBar from '@/views/NavBar.vue';
import { logService, LogLevel, ApiLogAppender, ConsoleLogAppender } from '@/services/logging';
import usersStore from '@/store-modules/user';
import { User } from '@/models';


@Component({
  components: {
    NavBar
  }
})
export default class App extends Vue {

  private consoleLogAppender: ConsoleLogAppender;
  private apiLogAppender: ApiLogAppender;

  constructor() {
    super();

    // Setup application logging.
    // TODO: prod/dev config settings for logging?
    this.consoleLogAppender = new ConsoleLogAppender(LogLevel.ALL);

    this.apiLogAppender = new ApiLogAppender(
      process.env.VUE_APP_PM_API_BASE + '/log/batch', '', LogLevel.WARN);
    this.apiLogAppender.batchSize = 1;
    this.apiLogAppender.minimumTimePassedInSec = 5;

    logService.ROOT_LOGGER.appenders.push(this.apiLogAppender, this.consoleLogAppender);

  }

  private get user(): User | null {
    return usersStore.user;
  }

  @Watch('user')
  private onUserChanged(val: User | null , oldVal: User | null) {
    if (val) { this.apiLogAppender.authToken = val.authToken; }
  }
}
