import {
  Action,
  getModule,
  Module,
  Mutation,
  MutationAction,
  VuexModule
} from 'vuex-module-decorators';
import { LoginSubmit, User } from '@/models';
import store from '@/store';
import api from '@/services/pm-api-client';
import { logService } from '@/services/logging';

@Module({ namespaced: true, name: 'user', store, dynamic: true })
class UserStoreModule extends VuexModule {
  public user: User | null = null;
  public users: User[] = [];

  private log = logService.getLogger('/store-modules/user');

  @Action({ commit: 'SET_USER', rawError: true })
  public async login(creds: LoginSubmit) {
    const authToken = await api.createAuthToken(creds);
    const user = await api.getUser(authToken);

    this.log.trace('User login successful.');
    api.setAuthToken(authToken);
    return user;
  }

  @MutationAction({ mutate: ['users'] })
  public async fetchAllUsers() {
    const users = await api.getAllUsers();
    return { users };
  }

  @MutationAction({ mutate: ['users'] })
  public async createUser(user: User) {
    const newUser = await api.createUser(user);
    return { users: this.users.concat([user]) };
  }

  @MutationAction({ mutate: ['users'] })
  public async deleteUser(userId: string) {
    await api.deleteUser(userId);
    return { users: this.users.filter((u) => u.id !== userId) };
  }

  @Mutation private SET_USER(user: User) { this.user = user; }
}

export default getModule(UserStoreModule);
